import pytest


def test_cafe_show_menu():
    """show_menuメソッドを実行するとカフェのメニューを文字列で返すこと"""
    # arrange
    from sample import Cafe, CafeMenuItem

    coffee = CafeMenuItem(name="コーヒー", price=400)
    spaghetti = CafeMenuItem(name="スパゲティ", price=800)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    cafe = Cafe()
    cafe.add_menu_item(coffee)
    cafe.add_menu_item(spaghetti)
    cafe.add_menu_item(icecream)

    expected = """\
コーヒー: 400円
スパゲティ: 800円
バニラアイス: 200円""".strip()

    # act
    actual = cafe.show_menu()

    # assert
    assert actual == expected


def test_calculate_price_coffee_1_ice_1():
    """コーヒー1アイス1を注文すると合計金額は600円であること"""
    # arrange
    from sample import CafeMenuItem, Order, calculate_price

    coffee = CafeMenuItem(name="コーヒー", price=400)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    orders = [
        Order(coffee, 1),
        Order(icecream, 1),
    ]
    expected = 600

    # act
    actual = calculate_price(orders)

    # assert
    assert actual == expected


def test_cafe_calculate_price_spagetti_1_ice_2():
    """スパゲティ1アイス2を注文すると合計金額は1200円であること"""
    # arrange
    from sample import CafeMenuItem, Order, calculate_price

    spaghetti = CafeMenuItem(name="スパゲティ", price=800)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    orders = [
        Order(spaghetti, 1),
        Order(icecream, 2),
    ]
    expected = 1200

    # act
    actual = calculate_price(orders)

    # assert
    assert actual == expected


def test_cafe_calculate_price_coffee_1_spagetti_1_ice_1_discounted():
    """コーヒ1スパゲティ1アイス1を注文すると割引されて合計金額は1200円であること"""
    # arrange
    from sample import CafeMenuItem, Order, calculate_price

    coffee = CafeMenuItem(name="コーヒー", price=400)
    spaghetti = CafeMenuItem(name="スパゲティ", price=800)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    orders = [
        Order(coffee, 1),
        Order(spaghetti, 1),
        Order(icecream, 1),
    ]
    expected = 1200

    # act
    actual = calculate_price(orders)

    # assert
    assert actual == expected


def test_cafe_calculate_price_coffee_3_spagetti_2():
    """コーヒー3スパゲティ2を注文すると合計金額は割引されて2600円であること"""
    # arrange
    from sample import CafeMenuItem, Order, calculate_price

    coffee = CafeMenuItem(name="コーヒー", price=400)
    spaghetti = CafeMenuItem(name="スパゲティ", price=800)

    orders = [
        Order(coffee, 3),
        Order(spaghetti, 2),
    ]
    expected = 2600

    # act
    actual = calculate_price(orders)

    # assert
    assert actual == expected
