import pytest


def test_cafe_menu():
    """show_menuメソッドを実行するとカフェのメニューを文字列で返すこと"""
    # arrange
    from sample import Cafe, CafeMenuItem

    coffee = CafeMenuItem(name="コーヒー", price=400)
    spaghetti = CafeMenuItem(name="スパゲティ", price=800)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    cafe = Cafe()
    cafe.add_menu_item(coffee)
    cafe.add_menu_item(spaghetti)
    cafe.add_menu_item(icecream)

    expected = """\
コーヒー: 400円
スパゲティ: 800円
バニラアイス: 200円"""

    # act
    actual = cafe.show_menu()

    # assert
    assert actual == expected


@pytest.mark.parametrize(
    "name, count, item_name",
    [
        ("コーヒー", 1, "coffee"),
        ("スパゲティ", 2, "spaghetti"),
    ],
)
def test_cafe_order(name, count, item_name):
    """商品名・数量を設定してオーダーするとentryオブジェクトに注文が追加されること"""
    # arrange
    from sample import Cafe, CafeMenuItem, Order

    coffee = CafeMenuItem(name="コーヒー", price=400)
    spaghetti = CafeMenuItem(name="スパゲティ", price=800)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    menu_item_dict = {
        "coffee": coffee,
        "spaghetti": spaghetti,
        "icecream": icecream,
    }

    cafe = Cafe()
    cafe.add_menu_item(coffee)
    cafe.add_menu_item(spaghetti)
    cafe.add_menu_item(icecream)
    entry = cafe.enter()

    expected = [Order(menu_item_dict[item_name], count)]

    # act
    cafe.order(entry, name, count)
    actual = entry.orders

    # assert
    assert actual == expected


@pytest.mark.parametrize(
    "names, counts, expected",
    [
        # コーヒー1アイス1を注文すると合計金額は600円であること
        (
            ["coffee", "icecream"],
            [1, 1],
            600,
        ),
        # スパゲティ1アイス2を注文すると合計金額は1200円であること
        (
            ["spaghetti", "icecream"],
            [1, 2],
            1200,
        ),
        # コーヒ1スパゲティ1アイス1を注文すると割引されて合計金額は1200円であること
        (
            ["coffee", "spaghetti", "icecream"],
            [1, 1, 1],
            1200,
        ),
        # コーヒー3スパゲティ2を注文すると合計金額は割引されて2600円であること
        (
            ["coffee", "spaghetti"],
            [3, 2],
            2600,
        ),
    ],
)
def test_calculate_price(names, counts, expected):
    """calculate_priceが商品・数量に応じて適切な金額を返すこと"""
    # arrange
    from sample import CafeMenuItem, Order, calculate_price

    coffee = CafeMenuItem(name="コーヒー", price=400)
    spaghetti = CafeMenuItem(name="スパゲティ", price=800)
    icecream = CafeMenuItem(name="バニラアイス", price=200)

    menu_item_dict = {
        "coffee": coffee,
        "spaghetti": spaghetti,
        "icecream": icecream,
    }

    orders = [Order(menu_item_dict[name], count) for name, count in zip(names, counts)]

    # act
    actual = calculate_price(orders)

    # assert
    assert actual == expected
