import pytest


@pytest.fixture
def coffee():
    from sample import CafeMenuItem

    return CafeMenuItem(name="コーヒー", price=400)


@pytest.fixture
def spaghetti():
    from sample import CafeMenuItem

    return CafeMenuItem(name="スパゲティ", price=800)


@pytest.fixture
def icecream():
    from sample import CafeMenuItem

    return CafeMenuItem(name="バニラアイス", price=200)


@pytest.fixture
def menu_item_dict(coffee, spaghetti, icecream):
    return {
        "coffee": coffee,
        "spaghetti": spaghetti,
        "icecream": icecream,
    }


class TestCafe:
    """メニュー追加済みのカフェインスタンスのテスト"""

    @pytest.fixture
    def target(self, coffee, spaghetti, icecream):
        from sample import Cafe

        cafe = Cafe()
        cafe.add_menu_item(coffee)
        cafe.add_menu_item(spaghetti)
        cafe.add_menu_item(icecream)
        return cafe

    def test_show_menu(self, target):
        """show_menuメソッドを実行するとカフェのメニューを文字列で返すこと"""

        expected = """\
コーヒー: 400円
スパゲティ: 800円
バニラアイス: 200円"""

        # act
        actual = target.show_menu()

        # assert
        assert actual == expected

    @pytest.mark.parametrize(
        "name, count, item_name",
        [
            ("コーヒー", 1, "coffee"),
            ("スパゲティ", 2, "spaghetti"),
        ],
    )
    def test_cafe_order(
        self,
        target,
        menu_item_dict,
        name,
        count,
        item_name,
    ):
        """商品名・数量を設定してオーダーするとentryオブジェクトに注文が追加されること"""
        # arrange
        from sample import Order

        entry = target.enter()
        expected = [Order(menu_item_dict[item_name], count)]

        # act
        target.order(entry, name, count)
        actual = entry.orders

        # assert
        assert actual == expected


class TestCalculatePrice:
    """calculate_price関数のテスト"""

    @pytest.fixture
    def target(self):
        from sample import calculate_price

        return calculate_price

    @pytest.mark.parametrize(
        "names, counts, expected",
        [
            # コーヒー1アイス1を注文すると
            # 合計金額は600円であること
            (
                ["coffee", "icecream"],
                [1, 1],
                600,
            ),
            # スパゲティ1アイス2を注文すると
            # 合計金額は1200円であること
            (
                ["spaghetti", "icecream"],
                [1, 2],
                1200,
            ),
            # コーヒ1スパゲティ1アイス1を注文すると
            # 割引されて合計金額は1200円であること
            (
                ["coffee", "spaghetti", "icecream"],
                [1, 1, 1],
                1200,
            ),
            # コーヒー3スパゲティ2を注文すると
            # 合計金額は割引されて2600円であること
            (
                ["coffee", "spaghetti"],
                [3, 2],
                2600,
            ),
        ],
    )
    def test_it(
        self,
        target,
        menu_item_dict,
        names,
        counts,
        expected,
    ):
        """商品・数量に応じて適切な金額を返すこと"""
        # arrange
        from sample import Order

        orders = [Order(menu_item_dict[name], count) for name, count in zip(names, counts)]

        # act
        actual = target(orders)

        # assert
        assert actual == expected
